@@ -213,6 +213,9 @@ tree_row_activated (GtkTreeView         *view,
 void
 greeter_language_handler (GreeterItemInfo *info, gpointer user_data)
 {
+  GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+  GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
   GtkWidget *view = NULL;
 
   if (dialog == NULL)
